/*
 *  Copyright (c) 2006 The Rohatyn Group.
 *
 *  All Rights Reserved.
 *
 *  This software is the proprietary information of The Rohatyn Group.
 *  Use is subject to license terms.
 */
package com.trg.play.rules;

import java.io.IOException;
import java.io.InputStreamReader;

import org.drools.RuleBase;
import org.drools.RuleBaseFactory;
import org.drools.WorkingMemory;
import org.drools.compiler.DroolsParserException;
import org.drools.compiler.PackageBuilder;
import org.drools.compiler.PackageBuilderErrors;
import org.drools.rule.Package;

/**
 * Simple credit check applications
 * 
 * Rules:
 * Credit approval criteria
 * 1. Applicant must has a name (no empty string)
 * 2. Applicant age is at least "18" 
 * 
 * Facts:
 * Mr Tudor, 35 years old
 * Tudor Jr, 15 years old
 * "", 99 years old
 * 
 * TODO : figure out how does DSL 'OR/XOR' syntax work
 * @author Andy Mei
 * @version $Id$
 */
public class PersonTest
{

	/**
	 * Simple Drools test  
	 * @param args
	 * @throws IOException 
	 * @throws DroolsParserException 
	 */
	public static void main(String[] args) throws Exception
	{
		//create a RuleBase
		RuleBase ruleBase = readRule();

		//create facts
		Person adult = new Person(1, "Mr Tudor", 35);
		Person child = new Person(2, "Tudor Jr", 15);
		Person noNamePerson = new Person(3, "", 99);

		WorkingMemory workingMemory = ruleBase.newStatefulSession();

		workingMemory.insert(adult);
		workingMemory.insert(child);
		workingMemory.insert(noNamePerson);

		workingMemory.fireAllRules();
	}

	/**
	 * Please note that this is the "low level" rule assembly API.
	 */
	private static RuleBase readRule() throws Exception
	{

		//optionally read in the DSL (if you are using it).
		//Reader dsl = new InputStreamReader( DroolsTest.class.getResourceAsStream( "/mylang.dsl" ) );

		//Use package builder to build up a rule package.
		//An alternative lower level class called "DrlParser" can also be used...

		PackageBuilder builder = new PackageBuilder();

		//this wil parse and compile in one step
		//NOTE: There are 2 methods here, the one argument one is for normal DRL.
		builder.addPackageFromDrl(new InputStreamReader(PersonTest.class
				.getResourceAsStream("/credit.drl")), new InputStreamReader(
				PersonTest.class.getResourceAsStream("/credit.dsl")));

		//check for build error
		PackageBuilderErrors errors = builder.getErrors();
		if ( errors.getErrors() != null && errors.getErrors().length != 0 )
		{
			System.err.println( "Package build error found:");
			System.err.println(errors);
		}
		//add ruleflow
//		builder.addRuleFlow(new InputStreamReader( PersonTest.class.getResourceAsStream( "/ruleflow.rfm" ) ));

		//get the compiled package (which is serializable)
		Package pkg = builder.getPackage();

		//add the package to a rulebase (deploy the rule package).
		RuleBase ruleBase = RuleBaseFactory.newRuleBase();
		
		ruleBase.addPackage(pkg);
		return ruleBase;
	}
}
